@@ -1,6 +1,9 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <pwd.h>
+#include <grp.h>
+#include <shadow.h>
 
 #include <sys/socket.h>
 
@@ -32,6 +35,21 @@ int HandleWebPages(char *HTTPBuffer,char *HTTPOutputBuffer,int size,int http_lis
 			ArgPosition++;
 		}
 
+		if(!strcmp(HTTPBuffer,"/Users")) {
+			DisplayTextHeader(http_message_socket);
+			ShowUsers(http_message_socket);
+			return(-1);
+		} else if(!strcmp(HTTPBuffer,"/Groups")) {
+			DisplayTextHeader(http_message_socket);
+			ShowGroups(http_message_socket);
+			return(-1);
+		} else if(!strcmp(HTTPBuffer,"/GroupMembers")) {
+			DisplayTextHeader(http_message_socket);
+			ShowGroupMembers(http_message_socket);
+			return(-1);
+		}
+
+
 		returncode=DefaultHeader(HTTPBuffer,pBuffer,size);
 		length=strlen(HTTPOutputBuffer);
 		
@@ -85,9 +103,8 @@ int HandleWebPages(char *HTTPBuffer,char *HTTPOutputBuffer,int size,int http_lis
 
 int Status_Page(char *source, char *dest, int size)
 {
-	
 	strncpy(dest,"<HTML><BODY><H1><CENTER>Welcome to SNARE for Solaris version 2.0</H1><P>Please select from the menu on the left.</CENTER><p>",size);
-	
+
 	// Ok.. we're cheating here. The only way we can get here is if we're running.
 	strncat(dest,"Status: <font color=green>Snare for Solaris currently running.</font>",size-strlen(dest));
 	
@@ -95,6 +112,112 @@ int Status_Page(char *source, char *dest, int size)
 	return(0);
 }
 
+int ShowUsers(int http_socket)
+{
+	struct passwd *user;
+	struct spwd *sp;
+	int retval;
+	char temp[1024];
+	char hostid[MAX_HOSTID];
+
+	getfqdn(hostid);
+
+	snprintf(temp,sizeof(temp),"HOST: %s\n",hostid);
+	retval=send(http_socket,temp,strlen(temp),0);
+	setpwent();
+	while((user=(struct passwd *)getpwent()) != (struct passwd *)NULL) {
+		snprintf(temp,sizeof(temp),"%s\t%d\t%s",user->pw_name,user->pw_uid,user->pw_gecos);
+		retval=send(http_socket,temp,strlen(temp),0);
+
+		if((sp = getspnam(user->pw_name)) != (struct spwd*)NULL) {
+			if(sp->sp_max > 0 && ((time(NULL) /86400) > (sp->sp_lstchg + sp->sp_max))) {
+
+				strncpy(temp,"\tPASSWD_EXPIRED",sizeof(temp));
+				retval=send(http_socket,temp,strlen(temp),0);
+			}
+			if(sp->sp_lstchg > 0 && sp->sp_max > 0 && sp->sp_lstchg > 0 && ((time(NULL) /86400) > (sp->sp_lstchg + sp->sp_max + sp->sp_inact))) {
+
+				strncpy(temp,"\tACCOUNT_INACTIVE",sizeof(temp));
+				retval=send(http_socket,temp,strlen(temp),0);
+			}
+
+			if(sp->sp_expire > 0 && (time(NULL) / 86400) > sp->sp_expire) {
+				strncpy(temp,"\tACCOUNT_EXPIRED",sizeof(temp));
+				retval=send(http_socket,temp,strlen(temp),0);
+			}
+			if(!strcmp(sp->sp_pwdp,"NP")) {
+				strncpy(temp,"\tACCOUNT_DISABLED",sizeof(temp));
+				retval=send(http_socket,temp,strlen(temp),0);
+			} else if(!strcmp(sp->sp_pwdp,"*LK*")) {
+				strncpy(temp,"\tACCOUNT_LOCKED",sizeof(temp));
+				retval=send(http_socket,temp,strlen(temp),0);
+			} else if(!strcmp(sp->sp_pwdp,"")) {
+				strncpy(temp,"\tNO_PASSWD",sizeof(temp));
+				retval=send(http_socket,temp,strlen(temp),0);
+			}
+
+		}
+		retval=send(http_socket,"\n",strlen("\n"),0);
+	}
+	endpwent();
+	
+	return(1);
+}
+
+int ShowGroups(int http_socket)
+{
+	struct group *groupp;
+	int retval;
+	char temp[1024];
+	char hostid[MAX_HOSTID];
+
+	getfqdn(hostid);
+
+	snprintf(temp,sizeof(temp),"HOST: %s\n",hostid);
+	retval=send(http_socket,temp,strlen(temp),0);
+	setgrent();
+	while((groupp=(struct group *)getgrent()) != (struct group *)NULL) {
+		snprintf(temp,1024,"%s\t%d\n",groupp->gr_name,groupp->gr_gid);
+		retval=send(http_socket,temp,strlen(temp),0);
+	}
+	endgrent();
+	
+	return(1);
+}
+
+int ShowGroupMembers(int http_socket)
+{
+	struct group *groupp;
+	int retval;
+	char temp[1024];
+	int firstmember=1;
+	char hostid[MAX_HOSTID];
+
+	getfqdn(hostid);
+
+	snprintf(temp,sizeof(temp),"HOST: %s\n",hostid);
+	retval=send(http_socket,temp,strlen(temp),0);
+	setgrent();
+	while((groupp=(struct group *)getgrent()) != (struct group *)NULL) {
+		firstmember=1;
+		snprintf(temp,sizeof(temp),"%s\t%d\t",groupp->gr_name,groupp->gr_gid);
+		retval=send(http_socket,temp,strlen(temp),0);
+		while(*(groupp->gr_mem) != NULL) {
+			if(firstmember) {
+            			snprintf(temp,sizeof(temp),"%s",*((groupp->gr_mem)++));
+				firstmember=0;
+			} else {
+            			snprintf(temp,sizeof(temp),",%s",*((groupp->gr_mem)++));
+			}
+			retval=send(http_socket,temp,strlen(temp),0);
+		}
+		retval=send(http_socket,"\n",strlen("\n"),0);
+	}
+	endgrent();
+	
+	return(1);
+}
+
 int Network_Config(char *source, char *dest, int size)
 {
 	struct Reg_Config config_struct;
@@ -1424,7 +1547,10 @@ int DefaultHeader(char *source, char *dest, int size)
       "<br><A HREF=\"/remote\" style=\"color:FFFFFF;text-decoration:none\">Remote Control Configuration</A><br>" \
       "<br><A HREF=\"/objective\" style=\"color:FFFFFF;text-decoration:none\">Objectives Configuration</A><br>" \
       "<br><A HREF=\"/status\" style=\"color:FFFFFF;text-decoration:none\">View Audit Service Status</A><br>" \
-	  "<br><A HREF=\"/restart\" style=\"color:FFFFFF;text-decoration:none\">Apply the Latest Audit Configuration</A><br>" \
+      "<br><A HREF=\"/restart\" style=\"color:FFFFFF;text-decoration:none\">Apply the Latest Audit Configuration</A><br>" \
+      "<br><br><font size=-2><A HREF=\"/Users\" style=\"color:FFFFFF;text-decoration:none\">Display a list of Users</A><br>" \
+      "<br><A HREF=\"/Groups\" style=\"color:FFFFFF;text-decoration:none\">Display a list of Groups</A><br>" \
+      "<br><A HREF=\"/GroupMembers\" style=\"color:FFFFFF;text-decoration:none\">Display a list of GroupMembers</A></font><br>" \
       "</b></font>" \
       "<br>" \
       "</div>" \
@@ -1452,6 +1578,18 @@ int DefaultFooter(char *source, char *dest, int size)
 	return(0);
 }
 
+int DisplayTextHeader(int http_socket)
+{
+        char HTTPBuffer[512];
+        // Overwrite HTTPBuffer with the header data.
+        strncpy(HTTPBuffer, "HTTP/1.0 200 OK\r\n" \
+                            "Server: SNARE/1.0\r\n" \
+                            "MIME-version: 1.0\r\n" \
+                            "Content-type: text/plain\r\n\r\n",
+                        sizeof(HTTPBuffer));
+        return(send(http_socket,HTTPBuffer,strlen(HTTPBuffer),0));
+}
+
 int Restart(char *source, char *dest, int size, int socketone, int sockettwo)
 {
 	int pid;
@@ -1808,3 +1946,4 @@ int Close_Objective_File(FILE *configfile)
 {
 	return(fclose(configfile));
 }
+
