@@ -94,6 +94,8 @@ struct Reg_Objective
 int		HandleWebPages(char *HTTPBuffer,char *HTTPOutputBuffer,int size,int http_listen_socket, int http_message_socket);
 int		Status_Page(char *source, char *dest, int size);
 int		DefaultHeader(char *source, char *dest, int size);
+int		DisplayTextHeader(int http_socket);
+
 int		Network_Config(char *source, char *dest, int size);
 int		Remote_Config(char *source, char *dest, int size);
 int		Remote_Set(char *source, char *dest, int size);
@@ -114,6 +116,9 @@ int		Close_Objective_File(FILE *configfile);
 int     ReadObjectives();
 void    DestroyList(void);
 int		debracket(char *source, char *dest, int length);
+int		ShowUsers(int http_socket);
+int		ShowGroups(int http_socket);
+int		ShowGroupMembers(int http_socket);
 
 
 // Stuff we use from webserver.h
