@@ -1,12 +1,12 @@
 /**********************************************************
  *
- * Snare for Solaris version 1.6
+ * Snare for Solaris version 2.1
  *
  * Author: InterSect Alliance Pty Ltd
  *
  * Copyright 2001-2003 InterSect Alliance Pty Ltd
  *
- * Last Modified: 24/1/2003
+ * Last Modified: 29/8/2003
  *
  * Available under the terms of the GNU Public Licence.
  * - See www.gnu.org
@@ -51,6 +51,11 @@
  *                  /etc/security/audit_control and /etc/security/audit_class
  *                  to be available, otherwise new processes will NOT
  *                  inherit audit class settings.
+ *      15/ 8/2003  Added the User / Groups / Group Members capabilities
+ *                  to the micro-embedded web server.
+ *                  Fixed header returns to work with older netscapes.
+ *      24/ 8/2003  Reduced CPU usage of snarecore by changing the way
+ *                  information is read from the praudit pipe.
  *
  **********************************************************
  *
@@ -86,6 +91,11 @@
 #include "snaresolaris.h"
 #include "webserver.h"
 
+////// DEBUG
+#include <stdarg.h>
+void DebugMsg(const char* pszFormat, ...);
+////// END DEBUG
+
 // Linked List Functions
 static Node *head, *currentnode;
 static int AuditDestination=AUDIT_TO_STDOUT;            // Send to STDOUT unless otherwise stated.
@@ -139,7 +149,7 @@ int main(int argc, char *argv[])
 	int bytesread;			/* number of bytes read from the pipe */
 	int daemon2praudit[2];		/* Pipe from the daemon to praudit */
 	int praudit2daemon[2];		/* Pipe from praudit back to the daemon */
-	char logbuffer[LOGBUFFERSIZE];	/* Holds the data from praudit */
+	char logbuffer[8192];	/* Holds the data from praudit */
 	int logbuffercount=0;		/* current position in the log buffer */
 	int startlogbuffer=0;		/* the starting point in the log buffer for general audit log information */
 	int error;
@@ -296,7 +306,6 @@ int main(int argc, char *argv[])
 	/* This is effectively a while(true) */
 	while(continueloop)
 	{
-// printf("LOOP!!!");
 		/* Ok. We should be just about ready to audit now. */
 		/* Lets tell the kernel. */
 	
@@ -382,7 +391,7 @@ int main(int argc, char *argv[])
 					kill(ppid,SIGTERM); waitpid(pid,NULL,NULL);
 					exit(1);
 				}
-				
+
 				// Close unused fildes.
 				close(daemon2praudit[0]);
 				close(daemon2praudit[1]);
@@ -390,6 +399,7 @@ int main(int argc, char *argv[])
 				close(praudit2daemon[1]);
 						
 				execlp("/usr/sbin/praudit", "praudit", "-l", "-d	", (char *)0);
+
 				// We should never actually get here
 				fprintf(stderr,"Could not exec praudit\n");
 				audit_off();
@@ -411,6 +421,8 @@ int main(int argc, char *argv[])
 					/* Note that the audit-to-pipe patch had
 					   better be installed before we make this
 					   system call, or the machine will halt! */
+
+
 					auditsvc(daemon2praudit[1],0);
 					error=errno;
 
@@ -445,6 +457,8 @@ int main(int argc, char *argv[])
 					} else if(error == EBADF) {
 						// Pipe has been closed.
 						// Try to take out the praudit process, just in case.
+
+
 						kill(pid2,SIGTERM); waitpid(pid2,NULL,NULL);
 						exit(1);
 	
@@ -466,42 +480,81 @@ int main(int argc, char *argv[])
 			}
 
 			logbuffercount=0;
-			while ((bytesread=read(praudit2daemon[0],(void*)cache,1) >0) && continueloop)
+			char *newline=logbuffer;
+			char *endbuffer = logbuffer+sizeof(logbuffer)-1;
+			char *bufferptr = logbuffer;
+			long spaceavail=sizeof(logbuffer)-1;
+			const long buffersize=sizeof(logbuffer)-1;
+
+			char username[MAX_USERNAME];
+			char searchterm[MAX_PATH];
+			char eventname[MAX_EVENTNAME];
+			char options[MAX_OPTIONS]="";
+			char *startrecord=logbuffer;
+			int returncode;
+
+			char *tbuffer;
+
+			// while ((bytesread=read(praudit2daemon[0],bufferptr,spaceavail) >0) && continueloop)
+			while (continueloop)
 			{
-				/* Buffer the text until we receive a newline */
-				if(logbuffercount<(LOGBUFFERSIZE-1)) {
-					/* Dont overstep our memory bounds */
-					logbuffer[logbuffercount]=cache[0];
-					logbuffercount++;
+				if(spaceavail==0) {
+					// Oh dear.. filled our buffer without seeing a newline. Must be filled with corrupt data from praudit.
+					// Clear it.
+					bufferptr=logbuffer;
+					spaceavail=buffersize;
 				}
-				if(cache[0] == '\n')
-				{
-					char username[MAX_USERNAME];
-					char searchterm[MAX_PATH];
-					char eventname[MAX_EVENTNAME];
-					char options[MAX_OPTIONS]="";
-					int returncode;
-					
-					logbuffer[logbuffercount]='\0';
+
+				bytesread=read(praudit2daemon[0],bufferptr,spaceavail);
+				if(!bytesread) {
+					break;
+				}
+
+				if(bytesread < 1) continue;
+
+				tbuffer=bufferptr+bytesread-1;
+				tbuffer++;*tbuffer=0;
+
+				// While there are more newlines in the buffer
+				startrecord=logbuffer;
+				do {
+					newline=strstr(startrecord,"\n");
+					if(!newline) { break; }
+
+					*newline='\0';
+					newline++;
+
 					/* received a newline from praudit. */
 					/* Time to send the udp packet out. */
 					/* make sure the newline is included */
-					
 					/* This is where we implement the SNARE event checker process */
-					returncode = GetSolDetails(logbuffer,username,searchterm,eventname,options);
+					returncode = GetSolDetails(startrecord,username,searchterm,eventname,options);
 					while((nodematch=CheckObjective(username,searchterm,eventname,options,returncode)) != (Node *) NULL) {
 
-						FormatDelimiters(logbuffer);
+						FormatDelimiters(startrecord);
 
-						if(sendevent(logbuffer,nodematch->criticality) != 0)
+						if(sendevent(startrecord,nodematch->criticality) != 0)
 						{
 							/* Report an error, but continue */
 							fprintf(stderr,"error sending log entry to destination\n");
 						}
 					}
-					logbuffercount=0;
+
+					startrecord=newline;
+				} while(newline);
+
+				if(startrecord != logbuffer) {
+					int copysize=bytesread-(startrecord-bufferptr);
+					memcpy(logbuffer,startrecord,copysize);
+					bufferptr=logbuffer+copysize;
+					*bufferptr=0;
+					spaceavail=endbuffer-bufferptr;
+				} else {
+					bufferptr+=bytesread;
+					spaceavail-=bytesread;
 				}
 			}
+
 			if(caught_kill) {
 				caught_kill=0;
 				/* Someone has terminated the parent process. */
@@ -530,6 +583,8 @@ int main(int argc, char *argv[])
 				/* All done */
 				return 0;
 			} else if(caught_pipe) {
+
+
 				/* Oh dear, our child process has been terminated. */
 				caught_pipe=0;
 				/* Kill the other child process (auditsvc),
@@ -821,6 +876,7 @@ int read_config_file(int shortread)
 
 								// Also, ensure this event is turned on
 								turn_event_on(eventnumber);
+
 							} else {
 								if(!strcmp(event,"*")) {
 									AddToList("AUDIT_ALL",user,path,criticality,"",excludeflag,returncode);
@@ -1985,3 +2041,19 @@ char * FormatDelimiters(char *eventbuffer)
 	}
 	return(eventbuffer);
 }
+
+void DebugMsg(const char* pszFormat, ...)
+{
+	char buf[8192]="";
+	FILE *fp;
+
+	va_list arglist;
+	va_start(arglist, pszFormat);
+	vsnprintf(&buf[strlen(buf)],8192-strlen(buf)-1,pszFormat,arglist);
+	va_end(arglist);
+
+	fp=fopen("/tmp/SNARE-OUT","a");
+	fputs(buf,fp);
+	fclose(fp);
+}
+
