@@ -99,6 +99,7 @@ int HandleConnect()
 	char HTTPOutputBuffer[MAX_HTTPBUFFER];
 	char *pos;
 	int size=0;
+	int header=1;
 
 	retval = recv(http_message_socket,HTTPBuffer,sizeof (HTTPBuffer),0 );
 	// NOTE: Should probably do something about requests that are bigger than
@@ -121,8 +122,10 @@ int HandleConnect()
 	pos=strstr(HTTPBuffer,"Authorization: Basic ");
 	if(strlen(SnarePass) && !pos) {
 		RequestAuth(HTTPOutputBuffer,MAX_HTTPBUFFER);
+		header=0;
 	} else if(strlen(SnarePass) && !MatchAuth(pos + strlen("Authorization: Basic "))) {
 		RequestAuth(HTTPOutputBuffer,MAX_HTTPBUFFER);
+		header=0;
 	} else {
 		// Don't care about anything after the GET
 		pos=strstr(HTTPBuffer,"\n");
@@ -152,14 +155,32 @@ int HandleConnect()
 		size=HandleWebPages(HTTPBuffer,HTTPOutputBuffer,MAX_HTTPBUFFER,http_listen_socket,http_message_socket);
 	}
 
-	if(size == 0 && HTTPOutputBuffer) {
+	if(size == -1) {
+		size=0;
+	} else if(size == 0 && HTTPOutputBuffer) {
 		size=strlen(HTTPOutputBuffer);
 	}
 
-	retval = send(http_message_socket,HTTPOutputBuffer,size,0);
+	if(size) {
+		// Overwrite HTTPBuffer with the header data.
+		strncpy(HTTPBuffer, "HTTP/1.0 200 OK\r\n" \
+                                    "Server: SNARE/1.0\r\n" \
+                                    "MIME-version: 1.0\r\n" \
+                                    "Content-type: text/html\r\n\r\n",
+                                    sizeof(HTTPBuffer));
 
-	if (retval == -1) {
-		perror("send() failed");
+                if(header) {
+                        retval = send(http_message_socket,HTTPBuffer,strlen(HTTPBuffer),0);
+			if (retval == -1) {
+				perror("send() failed");
+			}
+		}
+
+		retval = send(http_message_socket,HTTPOutputBuffer,size,0);
+
+		if (retval == -1) {
+			perror("send() failed");
+		}
 	}
 
 	close(http_message_socket);
